;************************************************************************
;* FILENAME:      ADCfir.ASM                                            *
;*                                                                      *
;* DESCRIPTION:   This program demonstrate how to implement a           *
;*                16 TAPFIR Filter                                      *
;*                                                                      *
;************************************************************************
;*                                                                      *
;* DATE:          01/15/2002                                            *
;* VERSION:       2.0                                                   *
;*                                                                      *
;* AUTHOR(s):     Francois Turgeon / Gilles Rust                        *
;*                Goal Semiconductor inc.                               *
;*                                                                      *
;************************************************************************

;***************************
;* CONSTANTS DECLARATION   *
;***************************

NPOINTS         EQU     016                 ;CONSTANT OF NUMBER OF POINT
NPOINTSBASEADRS EQU     080h                ;PREVIOUSLY ACQUIRED NPOINT TABLE
COEFBASEADRS    EQU     0C0h                ;COEFFICIENT TABLE START ADRESS

;************************************************************
;* V003 SPECIFICS SPECIALS FUNCTION REGISTERS DECLARATION   *
;************************************************************

IOCTRL          EQU     0BAh                ;IO CONTROL REGISTER
BGAPCAL         EQU     095h                ;BANDGAP REGISTER
INMUXCTRL       EQU     0FCh                ;INPUT MUX CONTROL REGISTER
PGACTRL         EQU     0F9h                ;PGA REGISTER
OUTMUXCTRL      EQU     0FDh                ;OUTPUT MUX CONTROL REGISTER
ADCLKDIV        EQU     0FEh                ;ADC CLOCK DIVIDER REGISTER
ADCTRL          EQU     094h                ;ADC CONTROL REGISTER
ADCALADRS       EQU     096h                ;ADC CALIBRATION ADRESS REGISTER
ADCONVLOW       EQU     0F5h                ;ADC CONVERSION REGISTER (LOW)
ADCONVMED       EQU     0F6h                ;ADC CONVERSION REGISTER (MEDIUM)
ADCONVHIGH      EQU     0F7h                ;ADC CONVERSION REGISTER (HIGH)
ADCH0L          EQU     0A4h                ;ADC CHANNEL 0 LOW REGISTER
ADCH0H          EQU     0A5h                ;ADC CHANNEL 0 HIGH REGISTER

SPICTRL         EQU     0B4h                ;SPI CONTROL REGISTER
SPIIRQMASK      EQU     0B7h                ;SPI IRQ MASK REGISTER
SPITX           EQU     0B6h                ;SPI TX REGISTER
SPIRX           EQU     0B5h                ;SPI RX REGISTER
IRQSOURCE       EQU     0E4h                ;SPI/J1708 IRQ SOURCE REGISTER
SPIINTSTATUS    EQU     0BCh                ;SPI IRQ STATUS REGISTER

MACA1           EQU     0E7h                ;MAC UNIT SFR DECLARATION
MACA0           EQU     0E6h

MACB1           EQU     0EFh
MACB0           EQU     0EEh

MACACC3         EQU     0C7h
MACACC2         EQU     0C6h
MACACC1         EQU     0C5h
MACACC0         EQU     0C4h

MACRES3         EQU     0EDh
MACRES2         EQU     0ECh
MACRES1         EQU     0EBh 
MACRES0         EQU     0EAh

;**********************************
;* PROGRAM VARIABLES DECLARATION  *
;**********************************

BINH            EQU     033h                ;VARIABLE MSB BINARY
BINL            EQU     034h                ;VARIABLE LSB BINARY

TMPL            EQU     035h                ;TEMP VARIABLE FOR BIN2BCD
TMPH            EQU     036h                ;TEMP VARIABLE FOR BIN2BCD

YNDATAH         EQU     038h                ;RESULT OF FIR COMPUTATION
YNDATAL         EQU     039h
CPTR            EQU     03Ah                ;GENERAL PURPOSE COUNTER

SPITXMOT        EQU     042h                ;VARIABLE FOR SPI TRANSMISSION

DATAH           EQU     040h                ;VARIABLES HOLDING TLV5616 VALUE
DATAL           EQU     041h
ZOUFH           EQU     042h
ZOUFL           EQU     043h

MSBADC          EQU     060h
LSBADC          EQU     061h
 
;*********************************
;* INTERRUPT VECTORS JUMP TABLE  *
;*********************************

                ORG     0000h
                LJMP    INISYSTEM           ;RESET JUMP
                ORG     0043h
                LJMP    IRQADC              ;V003 ADC INTERRUPT

;**************************
;* MAIN PROGRAM BEGINING  *
;**************************

                ORG     0100h

INISYSTEM:      CLR     DPS                 ;FORCER DATA POINTER 0
                MOV     SP,#060H            ;DEFINE STACK POINTER LOCATION
                MOV     ADCALADRS,#10h

                CLR     EA                  ;DISABLE ALL IRQ
                MOV     IOCTRL,#0Fh         ;IO CONFIGURED IN OUTPUT, HIGH

                MOV     DPTR,#0B01h         ;RETRIEVE CALIBRATION DATA FROM OTPR
                MOVX    A,@DPTR               
                MOV     BGAPCAL,A           ;CALIBRATION BANDGAP

                MOV     DPL,#03h            ;RETRIEVE CALIBRATION DATA FROM OTPR
                MOVX    A,@DPTR          
                MOV     INMUXCTRL,A         ;[7] BANDGAP ORDER CORRECTION

                MOV     DPL,#05h            ;RETRIEVE CALIBRATION DATA FROM OTPR
                MOVX    A,@DPTR
                MOV     PGACTRL,A           ;CALIBRATION PGA

                MOV     A,INMUXCTRL         ;[7] BANDGAP ORDER CORRECTION
                ORL     A,#0Fh              ;[6:4] ADC CONVERSION DONE ON
                                            ;1ST FOUR CHANNELS
                MOV     INMUXCTRL,A         ;[3:0] ACTIVATE 4 INPUT BUFFERS

                MOV     SPICTRL,#00110001b  ;NO CS, 2MHz, NORMAL POL, MASTER MODE

                MOV     SPIIRQMASK,#00h     ;NO IRQ SPI ENABLED

                MOV     OUTMUXCTRL,#19h     ;[4] POWER DOWN SPARE AMPS
                                            ;[3:1] BGI ROUTED TO TA
                                            ;[0] OUTMUX ENABLE

                MOV     ADCLKDIV,#0Fh       ;SET CLOCK 250kHz FOR ADC

                MOV     ADCONVHIGH,#00h     ;ADC RATE 480Hz = 3C00h
                MOV     ADCONVMED,#3Ch      
                MOV     ADCONVLOW,#00h      

                MOV     ADCTRL,#00001000b   ;CLEAR ADC INTERRUPT FLAG               
                MOV     ADCTRL,#11010101b   ;[7] SV0 ON XTVREF
                                            ;[4] ADC POWER DOWN
                                            ;[2] ENABLE IRQ
                                            ;[0] BANDGAP POWERDOWN

                LCALL   CLRUPRAM            ;CLEAR UPPER RAM
                LCALL   COPYCOEF            ;COPY FIR FILTER COEFFICIENTS
                                            ;TO UPPER IRAM           

                SETB    EIE.0               ;ENABLE IRQ ADC
                SETB    EA                  ;ENABLE IRQ

                ORL     ADCTRL,#00000010b   ;[1] CONT CONVERSION

                LJMP    $                   ;LOOP

;******************************************************************************
;**********<<<<<<<<< INTERRUPT ROUTINES RESERVED SECTION >>>>>>>>>>************  
;******************************************************************************

                ORG     02000h

;************************************************************************
;* IRQ ROUTINE:         IRQADC - ROUTINE FROM AD CONVERTION             *
;*                                                                      *
;* DESCRIPTION:         PERFORM THE FIR CALCULATIONS                    *
;*                                                                      *
;************************************************************************
;* DATE:                November 15, 2001                               *
;* VERSION:             1.0                                             *
;*                                                                      *
;* AUTHOR(s)            Francois Turgeon                                *
;*                                                                      *
;************************************************************************

IRQADC:         CLR     EA                  ;DISABLE IRQ

                MOV     DATAH,ADCH0H
                MOV     DATAL,ADCH0L

                LCALL   FIRCOMPUTE          ;PERFORM CALCULATION

                MOV     DATAH,SPIRX

                MOV     A,EXIF              ;CLEAR FLAG RAISED BY IRQ ADC
                CLR     ACC.4
                MOV     EXIF,A      

                SETB    EA                  ;ENABLE IRQ

                ORL     ADCTRL,#00001000b   ;CLEAR ADC FLAG

                RETI

;**********************************************************************************
;********* INDIVIDUAL FUNCTION USED BY THE MAIN PROGRAM AND IRQ ROUTINES **********
;**********************************************************************************

;************************************************************************
;* FILENAME:            CLRUPRAM.asm                                    *
;*                                                                      *
;* DESCRIPTION:         CLEAR UPPER 128 BYTES OF VERSA1 INTERNAL RAM    *
;*                                                                      *
;************************************************************************
;*                                                                      *
;* DATE:                November 15, 2001                               *
;* VERSION:             1.0                                             *
;*                                                                      *
;* AUTHOR(s)            Francois Turgeon                                *
;*                      Goal Semiconductor inc.                         *
;*                                                                      *
;************************************************************************

CLRUPRAM:       MOV     R0,#07FH            ;INIT RAM POINTER
CLRUPRAMLOOP:   INC     R0
                MOV     @R0,#00             ;CLEAR CURRENT UP RAM LOCATION
                CJNE    R0,#0FFH,CLRUPRAMLOOP ;CHECK IF ALL UPPER RAM LOCATION
                                              ;HAVE BEEN CLEARED
                RET

;************************************************************************
;* FILENAME:            COPYCOEF.asm                                    *
;*                                                                      *
;* DESCRIPTION:         COPY FIR COEFFICIENTS TO UPPER INTERNAL RAM     *
;*                      ADRESS: C0h - FFh                               *
;*                      USES NPOINT CONSTANT TO DEFINE HOW MANY COEF    *
;*                      TO COPY                                         *
;*                      COEFFICIENTS ARE STORED INTO IRAM IN THE        *
;*                      FOLLOWING FASHION:                              *
;*                                                                      *
;*                      IRAM ADRS     COEFFICIENT                       *
;*                                                                      *
;*                         C0h         COEF0 MSB                        *
;*                         C1h         COEF0 LSB                        *
;*                         C2h         COEF1 MSB                        *
;*                         C3h         COEF1 LSB                        *
;*                              (...)                                   *
;*                         XXh         COEFN MSB                        *
;*                         XYh         COEFN LSB                        *
;*                                                                      *
;*                      NOTE:  NPOINTS MAX VALUE = 32                   *
;*                                                                      *
;************************************************************************
;*                                                                      *
;* DATE:                November 15, 2001                               *
;* VERSION:             1.0                                             *
;*                                                                      *
;* AUTHOR(s)            Francois Turgeon                                *
;*                      Goal Semiconductor inc.                         *
;*                                                                      *
;************************************************************************

COPYCOEF:       MOV     DPTR,#COEFTABLE     ;SET POINTER TO COEF TABLE IN MEMORY
                MOV     CPTR,#NPOINTS       ;DEFINE GENERAL COUNTER
                MOV     R0,#0C0h            ;SET IRAM COEFFICIENT BASE ADRESS

COPYLOOP:       CLR     A                   ;CLEAR ACC 
                MOVC    A,@A+DPTR           ;GET COEFFICIENT UPPER 8 BITS
                MOV     @R0,A               ;STORE MSB OF COEFFICIENT
                INC     DPTR                ;POINT TO COEFFICIENT LSB VALUE
                INC     R0                  ;POINT TO LOCATION WHERE COEFFICIENT
                                            ;LSB WILL BE PUT
                CLR     A
                MOVC    A,@A+DPTR           ;GET COEFFICIENT LOWER 8 BITS
                MOV     @R0,A               ;STORE LSB OF COEFFICIENT
                INC     DPTR                ;POINT TO COEFFICIENT MSB VALUE
                INC     R0                  ;POINT TO LOCATION WHERE COEFFICIENT
                                            ;MSB WILL BE PUT
                DJNZ    CPTR,COPYLOOP       ;REPEAT UNTIL ALL COEFFICIENT ARE
                                            ;COPIED INTO UPPER IRAM
                RET

;************************************************************************
;* FILENAME:            FIRCOMPUTE.asm                                  *
;*                                                                      *
;* DESCRIPTION:         PERFORM FIR COMPUTATION USING MAC               *
;*                                                                      *
;************************************************************************
;*                                                                      *
;* DATE:                November 15, 2001                               *
;* VERSION:             1.0                                             *
;*                                                                      *
;* AUTHOR(s)            Francois Turgeon                                *
;*                      Goal Semiconductor inc.                         *
;*                                                                      *
;************************************************************************

FIRCOMPUTE:     MOV     R0,#NPOINTSBASEADRS ;INPUT ADC RAW DATA AT Xn LOCATIONS...
                MOV     ZOUFH,DATAH
                MOV     ZOUFL,DATAL

                MOV     @R0,ZOUFH           ;SAVE n0 ACQUIRED RAW DATA POINT (MSB)
                INC     R0
                MOV     @R0,ZOUFL           ;SAVE n0 ACQUIRED RAW DATA POINT (LSB)

                ;** PREPARE TO COMPUTE Yn...

                MOV     R0,#NPOINTSBASEADRS ;DEFINE BASE ADRS OF INPUTS VALUES
                MOV     R1,#COEFBASEADRS    ;DEFINE BASE ADRS OF COEFFICIENTS
                MOV     R7,#NPOINTS         ;DEFINE COUNTER

                CLR     A
                MOV     MACA1,A             ;INITIALISE MACA TO 0
                MOV     MACA0,A

                MOV     MACACC3,A           ;INITIALISE MACCAC TO 0
                MOV     MACACC2,A
                MOV     MACACC1,A
                MOV     MACACC0,A           ;BOTH COMBINED MAKE
                                            ;MACRESULT = 00000000h

               ;** COMPUTE Yn...

COMPUTELOOP:    MOV     MACACC3,MACRES3     ;COPY PREVIOUS CALCULATION RESULT
                MOV     MACACC2,MACRES2     ;INTO MACACC
                MOV     MACACC1,MACRES1
                MOV     MACACC0,MACRES0

                MOV     MACA1,@R0           ;PUT A GIVEN Xn INPUT INTO MACA
                INC     R0
                MOV     MACA0,@R0
                INC     R0

                MOV     MACB1,@R1           ;PUT A GIVEN COEFFICIENT INTO MACB
                INC     R1
                MOV     MACB0,@R1
                INC     R1

                DJNZ    R7,COMPUTELOOP      ;PERFORM COMPUTATION FOR N TAPS

               ;** SHIFT PREVIOUS INPUT VALUES TO LET PLACE FOR NEXT ONE...

;************************************************************************
;* FILENAME:            SHIFTPAST.asm                                   *
;*                                                                      *
;* DESCRIPTION:         SHIFT PAST INPUT VALUES                         *
;*                                                                      *
;************************************************************************
;*                                                                      *
;* DATE:                November 15, 2001                               *
;* VERSION:             1.0                                             *
;*                                                                      *
;* AUTHOR(s)            Francois Turgeon                                *
;*                      Goal Semiconductor inc.                         *
;*                      1134 Ste-Catherine West Suite 900               *
;*                      Montreal Quebec                                 *
;*                      Canada,  H3B-1H4                                *
;*                                                                      *
;*                      Tel: (514) 871-2447                             *
;*                      Fax: (514) 394-0661                             *
;*                      Web: www.goalasic.com                           *
;*                                                                      *
;************************************************************************

SHIFTPAST:      MOV     R7,#(NPOINTS-1)*2   ;DEFINE NUMBER OF DATASHIFT
                                            ;TO PERFORM ie (N-1)*2

                MOV     R0,#(NPOINTSBASEADRS - 1 + 2*(NPOINTS-1))
                                            ;COMPUTE FIRST FETCH ADDRESS
                MOV     R1,#(NPOINTSBASEADRS + 1 + 2*(NPOINTS-1))
                                            ;COMPUTE FIRST DESTINATION ADDRESS

SHIFTLOOP:      MOV     A,@R0               ;SHIFT GIVEN LSB INPUT...
                MOV     @R1,A               ;...TO NEXT LOCATION
                DEC     R0                  ;PREPARE POINTERS FOR MOVING LSB
                DEC     R1
                DJNZ    R7,SHIFTLOOP
                
               ;** PERFORM TRANSFORMATION OF Yn HERE AND PUT INTO BINH, BINL
               ;** IN THIS CASE THE COEFFICIENTS HAVE BEEN MULTIPLIED BY 65536
               ;** SO THE RESULT IS ON 32 BITS
               ;** DIVISING YN BY 65536 MEAN ONLY TAKING THE UPPER 16 BITS

                MOV     DATAH,MACRES3
                MOV     DATAL,MACRES2

                LCALL   TLV5616
                RET

;********************************************
;* FIR FILTER COEFFICIENTS TABLE DEFINITION *
;********************************************

;FSAMPLE 480HZ, N=16, LOW PASS 0.1HZ -78DB @ 60HZ

COEFTABLE:      DW      023DH
                DW      049DH
                DW      086AH
                DW      0D2DH
                DW      1263H
                DW      1752H
                DW      1B30H
                DW      1D51H
                DW      1D51H
                DW      1B30H
                DW      1752H
                DW      1263H
                DW      0D2DH
                DW      086AH
                DW      049DH
                DW      023DH

                DW      0FFFFH              ;END OF TABLE 

;************************************************************************
;* FILENAME:            TLV5616.asm                                     *
;*                                                                      *
;* DESCRIPTION:         TRANSMIT 12 BITS WORD TO TLV5616                *
;*                      12 BITS D/A CONVERTER                           *
;*                      THE FUNCTION TAKE CARE OF SETTING DEVICE IN     *
;*                      HIGH SPEED AND NORMAL POWER OPERATION           *
;*                                                                      *
;* INTERFACE:           SPI SCK IS CONNECTED TO TLV5616 CLK INPUT       *
;*                      SPI SDO IS CONNECTED TO TLV5616 DIN INPUT       *
;*                      IO1 IS CONNECTED TO TLV5616 FS INPUT            *
;*                      TLV5616 !CS INPUT IS CONNECTED TO GND           *
;*                                                                      *
;* INPUT PARAMETER:     DATAH                                           *
;*                      DATAL                                           *
;*                                                                      *
;************************************************************************
;*                                                                      *
;* DATE:                November 15, 2001                               *
;* VERSION:             1.0                                             *
;*                                                                      *
;* AUTHOR(s)            Francois Turgeon                                *
;*                      Goal Semiconductor inc.                         *
;*                                                                      *
;************************************************************************

TLV5616:        MOV     IOCTRL,#0Fh         ;FORCE IO1 BEING AN OUTPUT HIGH
                MOV     IOCTRL,#0Bh         ;FORCE HIGH->LOW TRANSITION ON FS PIN
                MOV     A,DATAH             ;GET MSB OF DATA TO SEND TO DAC
                ANL     A,#0FH              ;SET BITS ASSOCIATED WITH
                                            ;SPEED CONTROL = HIGH, PWR= NORMAL
                ORL     A,#01000000b
                MOV     SPITX,A             ;SEND OVER SPI
                MOV     SPITX,DATAL         ;GET AND SEND ON SPI LSB OF DATA
SPIEND:         MOV     A,SPIINTSTATUS
                JNB     ACC.0,SPIEND        ;WAIT TILL LAST TRANSMISSION COMPLETED 
                MOV     IOCTRL,#0Fh         ;FS 0->1 
                RET
END